#include "dict.h"

#include <KCodecs>
#include <KLocale>
#include <QTcpSocket>
#include <QTimer>
#include <QRegExp>
#include <QDebug>

const char *DictClient = "Dikt 1";

const char *Dict::DefaultHost = "dict.org";
const int Dict::DefaultPort = 2628;
const char *Dict::DefaultDatabase = "!";
const char *Dict::DefaultStrategy = ".";

Dict::Dict(const QString &host, int port)
	: socket(0), state(Disconnected)
{
	dict.host = host;
	dict.port = port;
	socket = new QTcpSocket(this);
	setupSocket();
	setupTimers();
}

void Dict::reset()
{
	delete socket;
	socket = new QTcpSocket(this);
	state = Disconnected;
	setupSocket();
}

void Dict::setupSocket()
{
	connect(socket, SIGNAL(readyRead()), SLOT(read()));
	connect(socket, SIGNAL(stateChanged(QAbstractSocket::SocketState)),
		SLOT(socketState()));
	connect(socket, SIGNAL(error(QAbstractSocket::SocketError)),
		SLOT(error()));
}

void Dict::setupTimers()
{
	setTimeouts(20000, 15000, 240000);
	connect(&connectTimer, SIGNAL(timeout()), SLOT(connectTimeout()));
	connect(&readTimer, SIGNAL(timeout()), SLOT(readTimeout()));
	connect(&idleTimer, SIGNAL(timeout()), SLOT(idleTimeout()));
}

void Dict::setTimeouts(int connect, int read, int idle)
{
	timeout.connect = connect;
	timeout.read = read;
	timeout.idle = idle;
}

void Dict::connectTimeout()
{
	connectTimer.stop();
	reset();
	request.clear();
	queue.clear();
	emit errorMessage(i18n("Connect Timeout"));
	emit statusMessage(i18n("Canceled"));
}

void Dict::readTimeout()
{
	readTimer.stop();
	socket->abort();
	request.clear();
	queue.clear();
	emit errorMessage(i18n("Read Timeout"));
	emit statusMessage(i18n("Canceled"));
}

void Dict::idleTimeout()
{
	send("quit");
}

void Dict::open()
{
	socket->connectToHost(dict.host, dict.port);
	connectTimer.start(timeout.connect);
}

void Dict::setHost(const QString &host, int port)
{
	if (dict.host != host || dict.port != port) {
		dict.host = host;
		dict.port = port;
		if (state >= Connected)
			socket->abort();
		else if (state == Connecting)
			reset();
		queue.clear();
		dict.ident.clear();
		dict.user.clear();
	}
}

void Dict::read()
{
	readTimer.stop();
	int bytes = socket->bytesAvailable();
	if (!bytes) return;

	if (state == Writing || state == Connected) state = Reading;
	emit statusMessage(i18n("Reading"));
	QByteArray buf = socket->read(bytes);

	// convert from utf only at newline
	// since a multibyte character can get split across packets
	int end = buf.lastIndexOf('\n');
	int len = buf.length();
	read(QString::fromUtf8(extra + buf.mid(0, end)).remove('\r'));

	if (len == end + 1) extra.clear();
	else extra = buf.mid(end + 1);
}

void Dict::send(const QString &command)
{
// 	qDebug() << "send" << command;

	if (state > Connected) {
		state = Writing;
		socket->write(command.toUtf8() + '\n');
		readTimer.start(timeout.read);
		idleTimer.start(timeout.idle);
		emit statusMessage(i18n("Sending"));
	} else {
		if (state == Disconnected)
			open();
		queue.append(command);
	}
}

void Dict::cancel()
{
	request.clear();
	queue.clear();
	connectTimer.stop();
	readTimer.stop();

	if (state > Connected) {
		state = Canceling;
		idleTimer.start();
	}
	else if (state == Connected)
		socket->abort();
	else if (state == Connecting)
		reset();

	emit statusMessage(i18n("Canceled"));
}

void Dict::status()
{
	request.type = Status;
	request.command = "status";
	send(request.command);
}

void Dict::define(const QString &query, const QString &database)
{
	Q_ASSERT(!database.isEmpty());

	request.type = Define;
	request.query = query;
	request.database = database;
	request.command = QString("define %1 \"%2\"")
		.arg(database).arg(request.query.remove('\"'));
	send(request.command);
}

void Dict::match(const QString &query, const QString &database, const QString &strategy)
{
	Q_ASSERT(!database.isEmpty());
	Q_ASSERT(!strategy.isEmpty());

	request.type = Match;
	request.query = query;
	request.database = database;
	request.strategy = strategy;
	request.command = QString("match %1 %2 \"%3\"")
		.arg(database).arg(strategy).arg(request.query.remove('\"'));
	send(request.command);
}

void Dict::info(const QString &query)
{
	if (query == "server")
		request.type = Server;
	else if (query == "databases")
		request.type = Databases;
	else if (query == "strategies")
		request.type = Strategies;
	else
		request.type = Info;

	request.command = QString("show %1").arg(query);
	send(request.command);
}

void Dict::setUser(const QString &username, const QString &password)
{
	dict.user = username;
	dict.password = password;
}

void Dict::auth(const QString &username, const QString &password)
{
	setUser(username, password);
	if (!dict.user.isEmpty()) {
		request.type = Auth;
		request.command = "";
		(state >= Connected)
			? auth() : open();
	}
}

void Dict::auth()
{
	KMD5 context(dict.msgid.toAscii());
	context.update(dict.password.toAscii());

	send(QString("auth %1 ").arg(dict.user) + context.hexDigest());
}

void Dict::serverIdent(const QString &text)
{
	QRegExp rx("(.*) <(.*)> (<.*>)");
	rx.indexIn(text);
	QStringList list = rx.capturedTexts();
	dict.ident = list[1];
	dict.capabilities = list[2].split('.');
	dict.msgid = list[3];
}

inline ResponseCode responseCode(const QString &line)
{
	return (ResponseCode)((line.size() >= 3) ? line.mid(0,3).toInt() : 0);
}

inline QString responseMessage(const QString &line)
{
	return (line.size() > 4) ? line.mid(4) : "";
}

inline bool endMark(const QString &line)
{
	return (line == ".");
}

void Dict::read(const QString &text)
{
	QStringList list = text.split('\n');
	int size = list.size();

	if (state == Reading) state = Parsing;

	for (int i = 0; i < size; i++) {
		QString line = list[i];
		if (state == Parsing) {
			if (!line.isEmpty())
				parse(line);
		} else if (state == Appending) {
			if (endMark(line)) {
				if (response.type == Words)
					response.text += '\n';
				state = Parsing;
			} else
				append(line);
		}
	}

	if (state == Parsing) state = Reading;
}

DictPair headpair(const QString &line)
{
	QString key = line.section('\"', 2, 2).trimmed();
	QString val = line.section('\"', 3, 3);
	return qMakePair(key, val);
}

void Dict::parse(const QString &line)
{
// 	qDebug() << line;
	ResponseCode code = responseCode(line);

	if (code != OK) {
		response.type = code;
		response.desc = responseMessage(line);
		emit responseReceived(response);
	}

	switch (code) {
	case Definitions:
// 		emit statusMessage(response.desc);
		break;
	case Words:
		state = Appending;
		break;
	case Matches:
		state = Appending;
		break;
	case OK:
		switch (response.type) {
		case Words:
			response.database = headpair(response.desc).second;
			break;
		case Matches:
			response.database = response.list[0].first;
			response.strategy = request.strategy;
			break;
		case DatabaseList:
		case StrategyList:
		case DatabaseInfo:
		case ServerInfo:
			break;
		case Hello:
		case Bye:
			return;
			break;
		default:
			qDebug() << "unhandled response" << response.type;
			return;
		}
		emit responseProcessed(response);
		emit responseText(response.text);
		emit statusMessage(i18n("Done"));
		response.clear();
		break;
	case NoMatch:
		response.text = i18n("No matches found");
		emit responseText(response.text);
		emit responseProcessed(response);
		emit statusMessage(i18n("Done"));
		response.clear();
		break;
	case DatabaseList:
	case StrategyList:
	case ServerInfo:
	case DatabaseInfo:
		state = Appending;
		break;
	case Hello:
		serverIdent(response.desc);
		send(QString("client \"%1\"").arg(DictClient));
		if (!dict.user.isEmpty())
			auth();
		if (!queue.isEmpty()) {
			foreach(QString command, queue)
				send(command);
			queue.clear();
		}
		break;
	case Bye:
		break;
	case StatusMessage:
		response.text = i18n("Connected to %1", dict.ident);
		emit responseProcessed(response);
		emit responseText(response.text);
		emit statusMessage(i18n("Done"));
		response.clear();
		break;
	case AuthOK:
		emit authenticated(true);
		if (request.type == Auth) {
		response.text = i18n("Authenticated to %1", dict.host);
		emit responseProcessed(response);
		emit responseText(response.text);
		emit statusMessage(i18n("Done"));
		}
		break;
	case AuthDenied:
		emit authenticated(false);
		if (request.type == Auth)
		emit errorMessage(i18n("Authentication failed"));
		break;
	case AccessDenied:
		emit errorMessage(i18n("Access denied"));
		break;
	case InvalidDatabase:
		emit errorMessage(i18n("Invalid database: %1", request.database));
		break;
	case InvalidStrategy:
		emit errorMessage(i18n("Invalid strategy: %1", request.strategy));
		break;
	case NoDatabase:
		emit errorMessage(i18n("No databases available"));
		break;
	case NoStrategy:
		emit errorMessage(i18n("No strategies available"));
		break;
	case ServerUnavailable:
		emit errorMessage(i18n("Server temporarily unavailable"));
		break;
	case ServerShutdown:
		emit errorMessage(i18n("Server shutting down"));
		break;
	default:
		qDebug() << "unhandled code" << code;
	}
}

QPair<QString, QString> pair(const QString &line)
{
	QString key = line.section(' ', 0, 0);
	QString val = line.section('\"', 1, 1);
	return qMakePair(key, val);
}

void Dict::append(const QString &line)
{
	switch (response.type) {
	case Words:
		response.text += line;
		break;
	case Matches:
		if (line.isEmpty()) return;
// 		response.text += line;
		response.list.append(pair(line));
		break;
	case DatabaseList:
		if (line.isEmpty()) return;
		response.list.append(pair(line));
		break;
	case StrategyList:
		if (line.isEmpty()) return;
		response.list.append(pair(line));
		break;
	case ServerInfo:
		if (line.isEmpty()) return;
		response.text += line;
		break;
	case DatabaseInfo:
		response.text += line;
		break;
	default:
		qDebug() << "unhandled code" << response.type;
	}
	if (!response.isEmpty()) response.text += '\n';
}

void Dict::socketState()
{
	QString message;
	switch (socket->state()) {
	case QAbstractSocket::UnconnectedState:
		readTimer.stop();
		idleTimer.stop();
		state = Disconnected;
		message = i18n("Disconnected");
		break;
	case QAbstractSocket::HostLookupState:
		state = HostLookup;
		message = i18n("Looking for %1...", dict.host);
		break;
	case QAbstractSocket::ConnectingState:
		state = Connecting;
		message = i18n("Connecting to %1...", dict.host);
		break;
	case QAbstractSocket::ConnectedState:
		connectTimer.stop();
		readTimer.start(timeout.read);
		state = Connected;
		message = i18n("Waiting for %1...", dict.host);
		break;
	case QAbstractSocket::ClosingState:
		message = i18n("Disconnecting");
		break;
	default:
		qDebug() << "unhandled state" << socket->state();
	}
	if (!message.isEmpty())
		emit statusMessage(message);

// 	qDebug() << "state" << socket->state();
}

void Dict::error()
{
	connectTimer.stop();
	QString message;
	switch (socket->error()) {
	case QAbstractSocket::RemoteHostClosedError:
		break;
	case QAbstractSocket::ConnectionRefusedError:
		message = i18n("Connection refused by %1", dict.host);
		break;
	case QAbstractSocket::HostNotFoundError:
		message = i18n("Host %1 not found", dict.host);
		break;
	default:
		message = socket->errorString();
// 		qDebug() << "unhandled error" << socket->error();
	}
	if (!message.isEmpty())
		emit errorMessage(message);

// 	qDebug() << "error" << socket->error();
}
