#ifndef DICT_H
#define DICT_H

#include <QPair>
#include <QList>
#include <QStringList>
#include <QTimer>
#include <QByteArray>

typedef QPair<QString, QString> DictPair;
typedef QList<DictPair> DictList;

class QTcpSocket;

enum RequestCommand {
	Define = 1,
	Match,
	Databases,
	Strategies,
	Info,
	Server,
	Auth,
	Status,
	Help,
	Quit
};

enum ResponseCode {
	DatabaseList = 110,
	StrategyList = 111,
	DatabaseInfo = 112,
	HelpMessage = 113,
	ServerInfo = 114,
	Definitions = 150,
	Words = 151,
	Matches = 152,

	StatusMessage = 210,
	Hello = 220,
	Bye = 221,
	AuthOK = 230,
	OK = 250,

	ServerUnavailable = 420,
	ServerShutdown = 421,

	SyntaxError = 500,
	IllegalParameter = 501,
	UnimplementedCommand = 502,
	UnimplementedParameter = 503,
	AccessDenied = 530,
	AuthDenied = 531,
	InvalidDatabase = 550,
	InvalidStrategy = 551,
	NoMatch = 552,
	NoDatabase = 554,
	NoStrategy = 555,
};

struct DictRequest {
	RequestCommand type;
	QString query;
	QString command;
	QString database;
	QString strategy;
	bool isEmpty() { return command.isEmpty(); };
	void clear() { command.clear(); type = RequestCommand(0); };
};

struct DictResponse {
	ResponseCode type;
	QString desc;
	QString text;
	QString database;
	QString strategy;
	DictList list;
	bool isEmpty() { return text.isEmpty(); };
	void clear() { text.clear(); type = ResponseCode(0); list.clear(); };
};

class Dict : public QObject
{
	Q_OBJECT

public:
	static const char *DefaultHost;
	static const int DefaultPort;
	static const char *DefaultDatabase;
	static const char *DefaultStrategy;

	Dict(const QString &host = DefaultHost, int port = DefaultPort);
	QString host() const { return dict.host; };
	int port() const { return dict.port; };
	void setTimeouts(int connect, int read, int idle);

	QString user() const { return dict.user; };
	QStringList capabilities() const { return dict.capabilities; };
	void cancel();
	void status();

public Q_SLOTS:
	void define(const QString &query, const QString &database = DefaultDatabase);
	void match(const QString &query, const QString &database = DefaultDatabase, const QString &strategy = DefaultStrategy);
	void info(const QString &query);
	void auth(const QString &username, const QString &password);
	void setHost(const QString &name, int port = DefaultPort);
	void setUser(const QString &name, const QString &password);

private Q_SLOTS:
	void read();
	void error();
	void socketState();
	void connectTimeout();
	void readTimeout();
	void idleTimeout();

Q_SIGNALS:
	void responseReceived(const DictResponse &);
	void responseProcessed(const DictResponse &);
	void responseText(const QString &);
	void connected(bool);
	void authenticated(bool);
	void statusMessage(const QString &);
	void errorMessage(const QString &);

private:
	enum DictState {
		Disconnected,
		HostLookup,
		Connecting,
		Connected,
		Writing,
		Reading,
		Parsing,
		Appending,
		Canceling,
	};
	QTcpSocket *socket;

	struct DictInfo {
		QString host;
		int port;
		QString user, password;
		QString ident, msgid;
		QStringList capabilities;
	} dict;

	struct DictTimeout {
		int connect, read, idle;
	} timeout;
	QTimer connectTimer, readTimer, idleTimer;

	DictState state;
	DictRequest request;
	DictResponse response;
	QStringList queue;
	QByteArray extra;

	void setupSocket();
	void setupTimers();
	void reset();

	void open();
	void auth();
	void serverIdent(const QString &text);
	void send(const QString &command);
	void read(const QString &text);
	void parse(const QString &text);
	void append(const QString &text);
};

#endif
