#include "dictpage.h"

#include <KLocale>
#include <QString>
#include <QStringList>
#include <QPalette>
#include <QDebug>

DictPage::DictPage(const DictResponse &response)
{
	set(response);
}

void DictPage::set(const DictResponse &response)
{
	switch (response.type) {
	case Words:
		words(response.text);
		break;
	case Matches:
		matches(response.list);
		break;
	case DatabaseList:
		databases(response.list);
		break;
	case StrategyList:
		strategies(response.list);
		break;
	case DatabaseInfo:
		words(response.text);
		break;
	case ServerInfo:
		server(response.text);
		break;
	default:
		set(response.text);
	}
}

QString DictPage::htmltext()
{
	return html;
}

void DictPage::set(const QString &text)
{
	html = text;
}

void DictPage::matches(const QString &text, bool suggestions)
{
	QString message = i18n("No matches found for %1", text);
	if (suggestions)
	html.prepend(QString("<div class=\"message\">%1. Suggestions:</div>").arg(message));
	else
	set(QString("<div class=\"message\">%1.</div>").arg(message));
}

void DictPage::heading2(const QString &text)
{
	html.prepend("<div class=\"head2\">" + text + ": ");
	html.append("</div>");
}

void DictPage::heading(const QString &text)
{
	html.prepend("<div class=\"head\">" + text + "</div>");
}

void DictPage::words(const QString &text)
{
	html = text;
	// replace html tags
	html.replace('<', "&lt;");
	html.replace('>', "&gt;");
	html.prepend("<div class=\"define\">");
	
	int start = 0;
	while ((start = html.indexOf('{', start + 1)) >= 0) {
		int end = html.indexOf('}', start);
		// missing matching brace
		if (end < 0) break;

		int len = end - start;
		// too long for a link
		if (len > 240) continue;

		QString link = html.mid(start + 1, len - 1);
		// multiple lines inside braces, could be source code
		if (link.count('\n') > 1) continue;

		int pos;
		// skip external links
		if ((pos = link.indexOf('(')) >= 0 && link.indexOf(')') > pos) {
			html.replace(start, len + 1, link);
			continue;
		}
		// format link
		html.replace(start, len + 1, QString("<a href=\"%1\">%2</a>")
			.arg(link.simplified()).arg(link));
	}
	html.append("</div>");
}

void DictPage::server(const QString &text)
{
	html = "<div class=\"info\">";
	html += text;
	html += "</div>";
}

void DictPage::matches(const DictList &list)
{
	html = "<div class=\"match\">";
	int count = qMin(list.size(), 240);
	for (int i = 0; i < count; i++)
		html += QString("<a href=\"%1\">%1</a>\n")
			.arg(list[i].second);
	html += "</div>";
}

void DictPage::suggestions(const DictList &list)
{
	html = "<span class=\"suggestion\">";
	int count = qMin(list.size(), 12);
	for (int i = 0; i < count; i++)
		html += QString("<a href=\"%1\">%1</a>, ")
			.arg(list[i].second);
	html.chop(2);
	html += "</span>";
}

void DictPage::databases(const DictList &list)
{
	html = "<table>";
	foreach (DictPair pair, list) {
		html += QString("<tr><td class=\"key\"><a href=\"about:%1\">%1</a><td>")
			.arg(pair.first);
		html += "<td>" + pair.second;
	}
	html += "</table>";
}

void DictPage::strategies(const DictList &list)
{
	html = "<table>";
	foreach (DictPair pair, list) {
		html += "<tr><td class=\"key\">" + pair.first;
		html += "<td>" + pair.second;
	}
	html += "</table>";
}

void DictPage::error(const QString &text, const QString &host)
{
	html = "<div class=\"error\">";
	html += QString("Error when connecting to %1").arg(host);
	html += QString("<div class=\"define\">%1</div>").arg(text);
	html += "</div>";
}

QString DictPage::stylesheet()
{
// 	QPalette palette;
	QString style;
	style += ".define { white-space: pre }";
	style += ".define { margin-top: 1em }";
	style += ".match { white-space: pre }";
	style += ".head { color: white; background: steelblue }";
// 	style += QString(".head { color: %1 }")
// 		.arg(palette.color(QPalette::HighlightedText).name());;
// 	style += QString(".head { background: %1 }")
// 		.arg(palette.color(QPalette::Highlight).name());
	style += ".info { font-family: mono }";
	style += ".info { white-space: pre }";
	style += ".key { padding-right: 1em }";
	style += ".head2 { margin-top: 1em; margin-bottom: 1em }";
	style += ".message { margin-bottom: 1em }";
	style += ".error { margin: 2em }";
	style += ".error { background: lightgrey }";
// 	style += QString(".error { background: %1 }")
// 		.arg(palette.color(QPalette::Background).name());
	style += "a { text-decoration: none }";
	return style;
}

void DictPage::about(const QString &text)
{
	html = "<div class=\"about\">";
	html += QString("<h2>%1</h2>").arg(text);
	html += "<p>Dikt is a dictionary client implementation of Dict protocol. Dikt is a network application, it looks up words on dict servers. You don't need to install any dictionaries, you only need a connection to Internet. You can find more about Dict protocol at <a href=\"http://www.dict.org/\">http://www.dict.org/</a>.";
	html += "<p>To look up a word, enter it into the query bar. By default Dikt searches dict.org dict server. To switch to a different server, enter the URL in the form <b>dict://host/word</b>, where <b>host</b> is the dict server to connect to, and <b>word</b> is the word to look for. You need to specify a URL only when you want to change the server, and only the first time. Dikt remembers previous hosts, and you can access them from the host menu.";
	html += "<p>You can find a list of dict servers at <a href=\"http://luetzschena-stahmeln.de/dictd/\">http://luetzschena-stahmeln.de/dictd/</a>.";
	html += "<p>Dikt is very similar to web browsers, and it should be equally easy to use. One option that needs some explanation is to yank selections. When it is set, Dikt will look up the text you select in your KDE and Qt applications.";
	html += "<p>Also like web browsers, Dikt doesn't have the control over the content it displays. Some dictionaries are misformatted, just like some web pages are. If you think that the problem is with Dikt, please report it over the Dikt web site <a href=\"http://diktv1.googlepages.com/\">http://diktv1.googlepages.com/</a>.";
	html += "<p>You can see this page again at any time by entering <b>about:</b> in the query bar.";
	html += "<p></div>";
}
