#include "dictwidgets.h"

#include <KComboBox>
#include <QToolBar>
#include <QDebug>

DictSelect::DictSelect(QObject *parent)
	: KSelectAction(parent), combobox(0) {}

QString DictSelect::currentData() const
{
	return (currentItem() >= 0 && actions().count())
		? currentAction()->data().toString() : 0;
}

void DictSelect::setCurrentData(const QString &data)
{
	foreach (QAction *action, actions())
		if (action->data().toString() == data) {
			setCurrentAction(action);
			if (combobox) combobox->setCurrentIndex(currentItem());
		}
}

void DictSelect::addData(const QString &text, const QString &data)
{
	addAction(text)->setData(data);
	if (combobox) combobox->addItem(data);
}

void DictSelect::setList(const DictList &list, const QString &selected)
{
	clear();
	foreach(DictPair action, list)
		addData(action.second, action.first);
	setCurrentData(selected);
	setEnabled(!list.isEmpty());
}

void DictSelect::clear()
{
	KSelectAction::clear();
	if (combobox) combobox->clear();
}

void DictSelect::actionTriggered(QAction *action)
{
	emit triggered(action->data().toString());
	emit triggered(currentItem());
}

QWidget *DictSelect::createWidget(QWidget *parent)
{
	QToolBar *toolbar = qobject_cast<QToolBar *>(parent);
	if (!toolbar)
		return KSelectAction::createWidget(parent);

	combobox = new KComboBox(toolbar);
	connect (combobox, SIGNAL(currentIndexChanged(const QString &)),
		SLOT(setCurrentData(const QString &)));
	connect (this, SIGNAL(triggered(int)),
		combobox, SLOT(setCurrentIndex(int)));

	int index = currentItem();
	foreach (QAction *action, actions())
		combobox->addItem(action->data().toString());
	combobox->setCurrentIndex(index);

	return combobox;
}


DictMenu::DictMenu(QObject *parent)
	: KActionMenu(parent)
{
	setDelayed(false);
	connect(menu(), SIGNAL(triggered(QAction *)),
		SLOT(actionTriggered(QAction *)));
}

QString DictMenu::text(const QString &data)
{
	foreach (QAction *action, menu()->actions())
		if (action->data().toString() == data)
			return action->text().remove('&');
	return 0;
}

void DictMenu::addData(const QString &text, const QString &data)
{
	QAction *action = menu()->addAction(text);
	action->setData(data);
	action->setCheckable(true);
}

void DictMenu::setList(const DictList &list, const QStringList &selected)
{
	clear();
	foreach(DictPair action, list)
		addData(action.second, action.first);
	setCurrentData(selected);
	setEnabled(!list.isEmpty());
}

void DictMenu::setCurrentData(const QStringList &list)
{
	foreach (QAction *action, menu()->actions())
		action->setChecked(list.contains(action->data().toString()));
}

QStringList DictMenu::currentData()
{
	QStringList list;
	foreach (QAction *action, menu()->actions())
		if (action->isChecked())
			list.append(action->data().toString());
	return list;
}

void DictMenu::actionTriggered(QAction *action)
{
	emit selected(currentData());
}


DictHistory::DictHistory(DictBrowser *parent, Direction dir)
	: QMenu(parent)
{
	browser = parent;
	direction = dir;
	connect(this, SIGNAL(triggered(QAction *)), SLOT(steps(QAction *)));
	connect(this, SIGNAL(aboutToShow()), SLOT(setMenu()));
}

void DictHistory::steps(QAction *action)
{
	browser->moveTo((direction) ? action->data().toInt() : -action->data().toInt());
}

void DictHistory::setMenu()
{
	setList((direction) ? browser->forwardList() : browser->backwardList());
}

void DictHistory::setList(const QStringList &list)
{
	clear();
	for (int i = 0; i < list.size(); i++)
		addAction(list.at(i))->setData(i + 1);
}


DictView::DictView(QWidget *parent)
	: QListWidget(parent)
{
	setSelectionMode(QAbstractItemView::MultiSelection);
	connect(this, SIGNAL(itemSelectionChanged()), SLOT(selection()));
}

void DictView::selection()
{
	QStringList list;
	for (int i = 0; i < count(); ++i)
		if (item(i)->isSelected())
			list << item(i)->data(Qt::UserRole).toString();
	emit selected(list);
}

void DictView::setList(const DictList &list, const QStringList &selected)
{
	blockSignals(true);
	clear();
	blockSignals(false);
	foreach (DictPair pair, list)
		addItem(pair.second);
	for (int i = 0; i < count(); ++i)
		item(i)->setData(Qt::UserRole, list[i].first);
	setCurrentData(selected);
}

QString DictView::data(int index)
{
	return item(index)->data(Qt::UserRole).toString();
}

void DictView::setCurrentData(const QStringList &list)
{
	blockSignals(true);
	for (int i = 0; i < count(); ++i)
		item(i)->setSelected(list.contains(data(i)));
	blockSignals(false);
}


#include <KLocale>
#include <QKeyEvent>

DictSearch::DictSearch(DictBrowser *parent)
	: QWidget(parent), browser(parent), found(false)
{
	setupUi(this);

	toolbutton->setIcon(KIcon("dialog-close"));
	label->setText(i18n("Find:"));
	connect(pattern, SIGNAL(textChanged(const QString &)),
		SLOT(search(const QString &)));

	layout()->setAlignment(Qt::AlignLeft);
	layout()->setContentsMargins(4, 0, 4, 0);
	setFocusProxy(pattern);
}

void DictSearch::show()
{
	QWidget::show();
	setFocus();
	pattern->selectAll();
}

void DictSearch::hide()
{
	browser->setFocus();
	QWidget::hide();
}

void DictSearch::find(QTextDocument::FindFlags backward)
{
	if (!found) {
		show();
		search(pattern->text());
		return;
	}
	browser->savePosition();
	if (backward && browser->textCursor().atStart())
		browser->moveCursor(QTextCursor::End);

	if (browser->find(pattern->text(), backward))
		message->setText(i18n("Text found"));
	else {
		browser->clearSelection();
		browser->moveCursor(QTextCursor::Start);
		browser->restorePosition();
		message->setText(i18n("No more matches"));
	}
}

void DictSearch::search(const QString &text)
{
	browser->savePosition();
	if (text.isEmpty()) {
		browser->clearSelection();
		browser->restorePosition();
		clear();
		return;
	}
	browser->moveCursor(QTextCursor::Start);
	if (found = browser->find(text))
		message->setText(i18n("Text found"));
	else {
		browser->restorePosition();
		message->setText(i18n("Text not found"));
	}
}

void DictSearch::clear()
{
	if (!isVisible())
		pattern->clear();
	message->clear();
	found = false;
}

void DictSearch::keyPressEvent(QKeyEvent *event)
{
	if (event->key() == Qt::Key_Escape)
		hide();
	else
		browser->keyPressEvent(event);
}


#include <KIconLoader>
#include <QPixmap>

DictStatus::DictStatus(QWidget *parent)
	: QWidget(parent)
{
	picture = new QLabel(this);
	label = new QLabel(this);
	QLayout *layout = new QHBoxLayout(this);
	layout->addWidget(picture);
	layout->addWidget(label);
	layout->setContentsMargins(4, 0, 10, 0);
}

void DictStatus::setStatus(const char *icon, const QString &text, bool enabled)
{
	QPixmap pix = SmallIcon(icon, KIconLoader::SizeSmall, (enabled) ? KIconLoader::DefaultState : KIconLoader::DisabledState);
	picture->setPixmap(pix);
	label->setText(text);
}
