#include "dictwindow.h"
#include "dictbrowser.h"
#include "dictwidgets.h"
#include "dictpage.h"
#include "dictsettings.h"
#include "dict.h"
 
#include <KApplication>
#include <KAction>
#include <KLocale>
#include <KActionCollection>
#include <KActionMenu>
#include <KStandardAction>
#include <KSelectAction>
#include <KToolBarPopupAction>
#include <KStatusBar>
#include <KMenuBar>
#include <KConfig>
#include <KConfigGroup>
#include <KPasswordDialog>
#include <KToolInvocation>

#include <QDebug>
#include <QComboBox>
#include <QLabel>
#include <QUrl>
#include <QSplitter>
#include <QStackedWidget>

DictWindow::DictWindow(QWidget *parent)
	: KXmlGuiWindow(parent), settings(0)
{
	entry = new QComboBox(this);
	entry->setEditable(true);
	entry->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
	entry->lineEdit()->setMaxLength(maxQueryLen);
	entry->setCompleter(0);

	browser = new DictBrowser(this);
	search = new DictSearch(browser);
	search->hide();

	databaseList = new DictView(this);
	matchList = new DictBrowser(this);
	matchList->setHistorySize(10);
	
	sidebar = new QStackedWidget(this);
	sidebar->addWidget(databaseList);
	sidebar->addWidget(matchList);

	splitter = new QSplitter(this);
	splitter->addWidget(browser);
	splitter->addWidget(sidebar);

	QWidget *widget = new QWidget(this);
	QVBoxLayout *layout = new QVBoxLayout(widget);
	layout->setContentsMargins(0, 0, 0, 0);
	layout->addWidget(splitter);
	layout->addWidget(search);
	setCentralWidget(widget);

	dict = new Dict();

	setupActions();
	setupGUI();
	setupBrowser();
	setupDict();
	setupStatusbar();
	readConfig();

	show();
}

DictWindow::~DictWindow()
{
	writeConfig();
	config->sync();
	delete dict;
}

void DictWindow::readConfig()
{
	config = KGlobal::config();
	KConfigGroup mainConfig(config, "Dikt");

	conf.autoMatch = mainConfig.readEntry("AutoMatch", false);
	conf.corrections = mainConfig.readEntry("Corrections", true);
	conf.headings = mainConfig.readEntry("Headings", true);
	conf.matchFrame = mainConfig.readEntry("MatchFrame", true);

	bool yank = mainConfig.readEntry("Yank", false);
	actionCollection()->action("yank")->setChecked(yank);
// 	yankMode(yank);
	enableMatchList(conf.matchFrame);

	QString font = mainConfig.readEntry("Font", "");
	if (!font.isEmpty()) {
		QFont newfont;
		if(newfont.fromString(font));
			setFont(newfont);
	}
	readWidthsConfig();
	readServerConfig();
}

void DictWindow::readServerConfig()
{
	KConfigGroup serverConfig(config, "Server");
	QStringList hosts = serverConfig.readEntry("Hosts", QStringList(Dict::DefaultHost));

	int selected = serverConfig.readEntry("Selected", -1);
	if (selected < 0) about("about:");
	else entry->lineEdit()->setFocus();
	selected = qBound(0, selected, hosts.size() - 1);
	QString host = hosts[selected];

	foreach(QString name, hosts)
		hostMenu->addAction(name);

	hostMenu->setCurrentItem(selected);
	setHost(host);
}

void DictWindow::hostConfig(const QString &host)
{
	KConfigGroup hostConfig(config, host);
	int port = hostConfig.readEntry("Port", Dict::DefaultPort);
	dict->setHost(host, port);

	QString username = hostConfig.readEntry("Username", "");
	QString password = hostConfig.readEntry("Password", "");
	if (!username.isEmpty())
		dict->setUser(username, password);
}

QStringList DictWindow::databaseConfig()
{
	KConfigGroup hostConfig(config, dict->host());
	return hostConfig.readEntry("Database", QStringList());
}

QString DictWindow::strategyConfig()
{
	KConfigGroup hostConfig(config, dict->host());
	return hostConfig.readEntry("Strategy", QString());
}

void DictWindow::writeConfig()
{
	KConfigGroup mainConfig(config, "Dikt");
	bool yank = actionCollection()->action("yank")->isChecked();
	mainConfig.writeEntry("Yank" , yank);
	mainConfig.writeEntry("Splitter", splitter->sizes());
	mainConfig.writeEntry("Sidebar", conf.sidebar);
	mainConfig.writeEntry("Stack", sidebar->currentIndex());

	mainConfig.writeEntry("AutoMatch", conf.autoMatch);
	mainConfig.writeEntry("Corrections", conf.corrections);
	mainConfig.writeEntry("Headings", conf.headings);
	mainConfig.writeEntry("MatchFrame", conf.matchFrame);

	mainConfig.writeEntry("Font", font().toString());

	writeServerConfig();
}

void DictWindow::writeServerConfig()
{
	KConfigGroup serverConfig(config, "Server");
	QStringList hosts = hostMenu->items();
	serverConfig.writeEntry("Hosts", hosts);
	serverConfig.writeEntry("Selected", hostMenu->currentItem());
	writeHostConfig();
}

void DictWindow::writeHostConfig()
{
	if (!databaseMenu->isEnabled())
		return;

	QString host = dict->host();
	KConfigGroup hostConf(config, host);
	hostConf.writeEntry("Database", database());
	hostConf.writeEntry("Strategy", strategy());
}

void DictWindow::readWidthsConfig()
{
	KConfigGroup mainConfig(config, "Dikt");
	int &side = conf.sidebar = mainConfig.readEntry("Sidebar", 0);
	if (side) side = 1;
	setSidebar(side);

	QList<int> sizes, widths;
	sizes << 0 << 1;
	widths << 100 << 200;
	sizes = mainConfig.readEntry("Splitter", sizes);
	widths = mainConfig.readEntry("Widths", widths);

	splitter->setSizes((sizes[side]) ? sizes: widths);
	sidebar->setVisible(sizes[side]);
	sidebar->setCurrentIndex(mainConfig.readEntry("Stack", 0));
}

void DictWindow::writeWidthsConfig()
{
	KConfigGroup mainConfig(config, "Dikt");
	QList<int> sizes = splitter->sizes();
	mainConfig.writeEntry("Widths", sizes);
}

void DictWindow::setupActions()
{
	KAction *text = new KAction(this);
	text->setText(i18n("Page Source"));
	text->setCheckable(true);
	actionCollection()->addAction("text", text);

	connect(text, SIGNAL(toggled(bool)), browser, SLOT(plainText(bool)));

	KAction *yank = new KAction(this);
	yank->setText(i18n("Yank Selections"));
	yank->setCheckable(true);
	actionCollection()->addAction("yank", yank);

	connect(yank, SIGNAL(toggled(bool)), SLOT(yankMode(bool)));

	KAction *sidebar = new KAction(this);
	sidebar->setText(i18n("Database List"));
// 	sidebar->setCheckable(true);
	actionCollection()->addAction("sidebar", sidebar);

	connect(sidebar, SIGNAL(triggered(bool)), SLOT(toggleDatabaseList()));

	KAction *matches = new KAction(this);
	matches->setText(i18n("Match List"));
// 	matches->setCheckable(true);
	actionCollection()->addAction("matches", matches);

	connect(matches, SIGNAL(triggered()), SLOT(toggleMatchList()));

	KAction *define = new KAction(this);
	define->setText(i18n("Define"));
	define->setShortcut(Qt::CTRL + Qt::Key_D);
	actionCollection()->addAction("define", define);

	connect(define, SIGNAL(triggered()), SLOT(define()));

	KAction *match = new KAction(this);
	match->setText(i18n("Match"));
	match->setShortcut(Qt::CTRL + Qt::Key_M);
	actionCollection()->addAction("match", match);

	connect(match, SIGNAL(triggered()), SLOT(match()));

	KAction *clear = new KAction(this);
	clear->setText(i18n("Clear History"));
	actionCollection()->addAction("clear", clear);
// 	clear->setEnabled(false);

	connect(clear, SIGNAL(triggered()), browser, SLOT(clearHistory()));
	connect(clear, SIGNAL(triggered()), matchList, SLOT(clearHistory()));

	KAction *prev = new KAction(this);
	prev->setText(i18n("Previous Match"));
	prev->setShortcut(Qt::CTRL + Qt::Key_Left);
	actionCollection()->addAction("prev", prev);
	prev->setEnabled(false);

	connect(prev, SIGNAL(triggered()), matchList, SLOT(backward()));
	connect(matchList, SIGNAL(backwardAvailable(bool)),
		prev, SLOT(setEnabled(bool)));

	KAction *next = new KAction(this);
	next->setText(i18n("Next Match"));
	next->setShortcut(Qt::CTRL + Qt::Key_Right);
	actionCollection()->addAction("next", next);
	next->setEnabled(false);

	connect(next, SIGNAL(triggered()), matchList, SLOT(forward()));
	connect(matchList, SIGNAL(forwardAvailable(bool)),
		next, SLOT(setEnabled(bool)));

	KActionMenu *info = new KActionMenu(this);
	info->setText(i18n("About"));
// 	info->setIcon(KIcon("dialog-information"));
	actionCollection()->addAction("info", info);

	infoMenu = info->menu();
 	connect(infoMenu, SIGNAL(triggered(QAction *)), SLOT(info(QAction *)));

	QAction *server = new QAction(this);
	server->setText(i18n("Server"));
	server->setData("server");
	infoMenu->addAction(server);

	QAction *databases = new QAction(this);
	databases->setText(i18n("Databases"));
	databases->setData("databases");
	infoMenu->addAction(databases);

	QAction *strategies = new QAction(this);
	strategies->setText(i18n("Strategies"));
	strategies->setData("strategies");
	infoMenu->addAction(strategies);

	hostMenu = new KSelectAction(this);
	hostMenu->setText(i18n("Host"));
	actionCollection()->addAction("host", hostMenu);

	connect(hostMenu, SIGNAL(triggered(const QString &)),
		SLOT(setHost(const QString &)));

	databaseMenu = new DictMenu(this);
	databaseMenu->setText(i18n("Database"));
	actionCollection()->addAction("database", databaseMenu);
	databaseMenu->setEnabled(false);

	connect(databaseMenu, SIGNAL(selected(const QStringList &)),
		databaseList, SLOT(setCurrentData(const QStringList &)));
	connect(databaseList, SIGNAL(selected(const QStringList &)),
		databaseMenu, SLOT(setCurrentData(const QStringList &)));

	strategyMenu = new DictSelect(this);
	strategyMenu->setText(i18n("Strategy"));
	actionCollection()->addAction("strategy", strategyMenu);
	strategyMenu->setEnabled(false);

	KAction *backward = new KToolBarPopupAction(KIcon("go-previous"),
		i18n("Backward"), this);
	actionCollection()->addAction("backward", backward);
	backward->setShortcut(Qt::ALT + Qt::Key_Left);
	backward->setEnabled(false);

	connect(backward, SIGNAL(triggered()), browser, SLOT(backward()));
	connect(browser, SIGNAL(backwardAvailable(bool)),
		backward, SLOT(setEnabled(bool)));
	connect(backward, SIGNAL(triggered()), SLOT(cancel()));

	backwardMenu = new DictHistory(browser, DictHistory::Backward);
	backward->setMenu(backwardMenu);
	connect(backwardMenu, SIGNAL(triggered(QAction *)), SLOT(cancel()));

	KAction *forward = new KToolBarPopupAction(KIcon("go-next"),
		i18n("Forward"), this);
	actionCollection()->addAction("forward", forward);
	forward->setShortcut(Qt::ALT + Qt::Key_Right);
	forward->setEnabled(false);

	connect(forward, SIGNAL(triggered()), browser, SLOT(forward()));
	connect(browser, SIGNAL(forwardAvailable(bool)),
		forward, SLOT(setEnabled(bool)));

	connect(forward, SIGNAL(triggered()), SLOT(cancel()));

	forwardMenu = new DictHistory(browser, DictHistory::Forward);
	forward->setMenu(forwardMenu);
	connect(forwardMenu, SIGNAL(triggered(QAction *)), SLOT(cancel()));

	stop = new KAction(KIcon("process-stop"), i18n("Stop"), this);
	stop->setShortcut(Qt::Key_Escape);
	actionCollection()->addAction("stop", stop);
	stop->setEnabled(false);

	connect(this, SIGNAL(loading(bool)), stop, SLOT(setEnabled(bool)));
	connect(this, SIGNAL(loading(bool)), matchList, SLOT(busyCursor(bool)));
	connect(this, SIGNAL(loading(bool)), browser, SLOT(busyCursor(bool)));
	connect(stop, SIGNAL(triggered()), SLOT(cancel()));

	KAction *frame = new KAction(this);
	frame->setText(i18n("Next Frame"));
	actionCollection()->addAction("frame", frame);

	connect(frame, SIGNAL(triggered()), SLOT(nextFrame()));

	KAction *auth = new KAction(this);
	auth->setText(i18n("Authentication..."));
	actionCollection()->addAction("auth", auth);

	connect(auth, SIGNAL(triggered()), SLOT(authenticate()));

	QLabel *label = new QLabel(this);
	label->setText(i18n("Look for:"));
	label->setIndent(10);
	KAction *action = new KAction(this);
	action->setDefaultWidget(label);
	action->setText(i18n("Look for"));
	actionCollection()->addAction("search", action);

	KAction* combo = new KAction(this);
	combo->setText(i18n("Query"));
	combo->setShortcut(Qt::CTRL + Qt::Key_L);
	combo->setDefaultWidget(entry);
	actionCollection()->addAction("query", combo);

	connect(combo, SIGNAL(triggered()), entry, SLOT(setFocus()));
	connect(combo, SIGNAL(triggered()), entry->lineEdit(), SLOT(selectAll()));
	connect(entry, SIGNAL(activated(int)), SLOT(define()));

	KStandardAction::openNew(this, SLOT(newWindow()), actionCollection());
// 	KStandardAction::save(this, SLOT(save()), actionCollection());
//  	KStandardAction::print(this, SLOT(print()), actionCollection());

	KStandardAction::close(this, SLOT(close()), actionCollection());
	KStandardAction::quit(kapp, SLOT(closeAllWindows()), actionCollection());

	KStandardAction::find(search, SLOT(show()), actionCollection());
	KStandardAction::findNext(search, SLOT(findNext()), actionCollection());
	KStandardAction::findPrev(search, SLOT(findPrev()), actionCollection());

	KStandardAction::showMenubar(this, SLOT(toggleMenubar()), actionCollection());
	KStandardAction::preferences(this, SLOT(configure()), actionCollection());
}

void DictWindow::setupBrowser()
{
	connect(browser, SIGNAL(historyChanged(const QString &)),
		entry, SLOT(setEditText(const QString &)));
	connect(browser, SIGNAL(historyChanged(const QString &)),
		SLOT(setCaption(const QString &)));

	connect(browser, SIGNAL(anchorClicked(const QUrl &)),
		SLOT(define(const QUrl &)));
	connect(browser, SIGNAL(highlighted(const QString &)),
		statusBar(), SLOT(showMessage(const QString &)));

	connect(browser, SIGNAL(historyChanged(const QString &)),
		search, SLOT(clear()));

	connect(matchList, SIGNAL(anchorClicked(const QUrl &)),
		SLOT(define(const QUrl &)));
	connect(matchList, SIGNAL(highlighted(const QString &)),
		statusBar(), SLOT(showMessage(const QString &)));

	browser->setStyleSheet(page.stylesheet());
	matchList->setStyleSheet(page.stylesheet());
}

void DictWindow::statusMessage(const QString &message)
{
	statusBar()->showMessage(message, (message.endsWith("...")) ? 0 : 4000);
}

void DictWindow::setHostStatus(bool connected)
{
	hostStatus->setStatus("network-workgroup", dict->host(), connected);
}

void DictWindow::setUserStatus(bool authenticated)
{
	if (!userStatus)
		userStatus = new DictStatus(statusBar());
	if (authenticated) {
		if (userStatus->isVisible())
			return;
		userStatus->setStatus("user-identity", dict->user(), authenticated);
		statusBar()->addPermanentWidget(userStatus);
		statusBar()->addPermanentWidget(hostStatus);
 		userStatus->show();
	} else
		statusBar()->removeWidget(userStatus);
}

void DictWindow::setupStatusbar()
{
	hostStatus = new DictStatus(statusBar());
	statusBar()->addPermanentWidget(hostStatus);
	userStatus = 0;
}

void DictWindow::setupDict()
{
// 	connect(dict, SIGNAL(responseText(const QString &)),
// 		SLOT(response(const QString &)));
	connect(dict, SIGNAL(statusMessage(const QString &)),
		SLOT(statusMessage(const QString &)));
	connect(dict, SIGNAL(errorMessage(const QString &)),
		SLOT(error(const QString &)));

	connect(dict, SIGNAL(responseProcessed(const DictResponse &)),
		SLOT(response(const DictResponse &)));
	connect(dict, SIGNAL(authenticated(bool)),
		SLOT(setUserStatus(bool)));
}

void DictWindow::setDatabaseMenu(const DictList &list)
{
	QStringList selected = database();
	if (selected.isEmpty())
		selected = databaseConfig();
	databaseMenu->setList(list, selected);
	databaseList->setList(list, selected);
}

void DictWindow::setStrategyMenu(const DictList &list)
{
	QString selected = strategy();
	if (selected.isEmpty())
		selected = strategyConfig();
	strategyMenu->setList(list, selected);
}

void DictWindow::define(const QString &text)
{
	if (text.isEmpty()) return;
	query = text.simplified();
	query.truncate(maxQueryLen);

	if (stop->isEnabled())
		return;
	tracker.clear();
	browser->setFocus();
	emit loading(true);
	if (query.startsWith("about:"))
		about(query);
	else if (query.startsWith("dict:"))
		url(query);
	else if (query.startsWith("http:")) {
		KToolInvocation::invokeBrowser(text);
		emit loading(false);
	}
	else foreach(QString database, databases())
			dict->define(query, database);
}

void DictWindow::define()
{
	define(entry->currentText());
}

void DictWindow::define(const QUrl &link)
{
	define(link.toString());
}

void DictWindow::match(const QString &text)
{
	if (text.isEmpty()) return;
	query = text.simplified();
	query.truncate(maxQueryLen);

	if (query.startsWith("about:"))
		return;
	else if(query.startsWith("dict:"))
		return;
	else if (query.startsWith("http:"))
		return;

	if (stop->isEnabled())
		return;
	tracker.match = true;
	browser->setFocus();
	emit loading(true);
	foreach(QString database, databases())
		dict->match(query, database, strategies());
}

void DictWindow::match()
{
	match(entry->currentText());
}

void DictWindow::about(const QString &query)
{
	QString text = query.mid(6);
	if (text.isEmpty()) {
		page.about("Dikt Version 1");
		browser->setPage(query, page.htmltext());
		emit loading(false);
		return;
	}
	foreach(QAction *action, infoMenu->actions())
		if (action->data().toString() == text) {
			dict->info(text);
			return;
		}
	foreach(QAction *action, databaseMenu->menu()->actions())
		if (action->data().toString() == text) {
			dict->info("info " + text);
			return;
		}
	error(i18n("No such database %1", text));
}

void DictWindow::info(QAction *action)
{
	QString text = action->data().toString();
	query = "about:" + text;
	browser->setFocus();
	emit loading(true);
	dict->info(text);
}

void DictWindow::response(const QString &text)
{
	qDebug() << text;
}

void DictWindow::response(const DictResponse &response)
{
	DictBrowser *target = browser;
	page.set(response);

	if (response.type == Words) {
		if (conf.headings)
			page.heading(response.database);
	} 
	else if (response.type == Matches) {
		if (tracker.correct) {
			page.suggestions(response.list);
			page.heading2(response.database);
			if (!tracker.hits)
				page.matches(query, true);
		} else if (conf.matchFrame) {
			target = matchList;
			sidebar->show();
			sidebar->setCurrentWidget(matchList);
			matchList->setFocus();
		}
		if (conf.headings && !tracker.correct)
			page.heading(response.database);
	}
	else if (response.type == NoMatch) {
		if (++tracker.misses < dbcount()) {
			if (tracker.count() >= dbcount()) {
				tracker.clear();
				emit loading(false);
			}
			return;
		} else if (conf.corrections && !tracker.correct) {
			tracker.clear();
			tracker.correct = true;
			emit loading(false);
			match(query);
			return;
		}
		page.matches(query, false);
	}
	else if (response.type == DatabaseList) {
		setDatabaseMenu(response.list);
		if (tracker.init) {
			setHostStatus(true);
			return;
		}
	}
	else if (response.type == StrategyList) {
		setStrategyMenu(response.list);
		if (tracker.init) {
// 			emit loading(false);
			tracker.init = false;
			return;
		}
	}
	else if (response.type == AuthOK) {
		initDatabases();
	}

	if (!tracker.hits)
		target->setPage(query, page.htmltext());
	else
		target->appendPage(query, page.htmltext());

	if (response.type == Words || response.type == Matches) {
		++tracker.hits;
		if (tracker.count() >= dbcount()) {
			tracker.clear();
			emit loading(false);
		}
	} else {
		tracker.clear();
		emit loading(false);
	}
}
	
void DictWindow::error(const QString &text)
{
	page.error(text, dict->host());
	browser->setPage(query, page.htmltext());
	browser->setFocus();
	emit loading(false);
}

void DictWindow::cancel()
{
	if (stop->isEnabled()) {
		tracker.clear();
		dict->cancel();
		emit loading(false);
		tracker.init = false;
	}
}

void DictWindow::url(const QString &query)
{
	QUrl url(query);
	QString host = url.host();
// 	int port = url.port();
	if (host.isEmpty()) {
		error(i18n("Malformed Url"));
		return;
	}
	if (host != dict->host()) {
		setHost(host);
		int index = hostMenu->items().indexOf(host);
		if (index < 0) {
			index = hostMenu->items().size();
			hostMenu->addAction(host);
		}
		hostMenu->setCurrentItem(index);
	}
	QString path = url.path();
	emit loading(true);
	if (path.size() <= 1)
		dict->status();
// 	else	dict->define(path.mid(1));
	else foreach(QString database, databases())
			dict->define(path.mid(1), database);
}

void DictWindow::setHost(const QString &host)
{
	cancel();
	writeHostConfig();
	hostConfig(host);
	databaseMenu->clear();
	databaseList->clear();
	strategyMenu->clear();
	databaseMenu->setDisabled(true);
	strategyMenu->setDisabled(true);
	tracker.clear();
	initDatabases();
	setHostStatus(false);
	if (userStatus)
		setUserStatus(false);
}

void DictWindow::initDatabases()
{
	tracker.init = true;
	dict->info("databases");
	dict->info("strategies");
// 	emit loading(true);
}

void DictWindow::yankMode(bool toggle)
{
	if (toggle)
		connect(kapp->clipboard(), SIGNAL(changed(QClipboard::Mode)),
			SLOT(yankClipboard(QClipboard::Mode)));
	else
		kapp->clipboard()->disconnect(this);
}

void DictWindow::yankClipboard(QClipboard::Mode mode)
{
	switch (mode) {
	case QClipboard::Clipboard:
		match(kapp->clipboard()->text(mode));
		break;
	case QClipboard::Selection:
		define(kapp->clipboard()->text(mode));
		break;
	case QClipboard::FindBuffer:
// 		find(kapp->clipboard()->text(mode));
		break;
	}
}

void DictWindow::authenticate()
{
	KPasswordDialog auth(this, KPasswordDialog::ShowUsernameLine | KPasswordDialog::ShowKeepPassword);
	auth.setCaption(i18n("Dict Authentication"));
	auth.setPrompt(i18n("Enter your authentication for host %1", dict->host()));
	auth.setUsername(dict->user());
	if (auth.exec()) {
		dict->auth(auth.username(), auth.password());
		query.clear();
	}
	KConfigGroup hostConf(config, dict->host());
	if (auth.keepPassword()) {
		hostConf.writeEntry("Username", auth.username());
		hostConf.writeEntry("Password", auth.password());
	} else {
		hostConf.deleteEntry("Username");
		hostConf.deleteEntry("Password");
	}
}

void DictWindow::toggleMenubar()
{
	menuBar()->setVisible(!menuBar()->isVisible());
}

void DictWindow::setSidebar(QWidget *widget)
{
	if (sidebar->isVisible())
		writeWidthsConfig();
	else
		widget->setFocus();
	sidebar->setVisible(!widget->isVisible());
	sidebar->setCurrentWidget(widget);
	if (!sidebar->isVisible())
		browser->setFocus();
}

void DictWindow::setSidebar(int side)
{
	splitter->insertWidget(side, sidebar);
}

void DictWindow::toggleDatabaseList()
{
	setSidebar(databaseList);
}

void DictWindow::toggleMatchList()
{
	setSidebar(matchList);
}

void DictWindow::enableMatchList(bool enable)
{
	actionCollection()->action("matches")->setEnabled(enable);
	if (!enable) {
		actionCollection()->action("prev")->setEnabled(enable);
		actionCollection()->action("next")->setEnabled(enable);
		if (matchList->isVisible())
			sidebar->hide();
		matchList->clearHistory();
	}
}

void DictWindow::nextFrame()
{
	if (browser->hasFocus())
		if (sidebar->isVisible())
			sidebar->currentWidget()->setFocus();
		else if (search->isVisible())
			search->setFocus();
		else
			entry->setFocus();
	else
		browser->setFocus();
}

void DictWindow::configure()
{
	if (!settings) {
		settings = new DictSettings(this);
		connect(settings, SIGNAL(accepted()), SLOT(loadSettings()));
	}
	settings->setHosts(hostMenu->items());
	settings->setFont(font());
	settings->readConf(conf);
	settings->show();
}

void DictWindow::loadSettings()
{
	QStringList hosts = settings->hosts();
	if (hosts.isEmpty())
		hosts << Dict::DefaultHost;
	QString host = hostMenu->currentText();
	int index = hosts.indexOf(host);
	hostMenu->setItems(hosts);
	if (index < 0)
		index = 0;
	hostMenu->setCurrentItem(index);
	host = hostMenu->currentText();
	if (host != dict->host())
		setHost(host);

	setFont(settings->font());
	settings->writeConf(conf);

	setSidebar(conf.sidebar);
	enableMatchList(conf.matchFrame);
}
